import re
import unicodedata
import os


def loc_sub(text, key, value):
    # Find the keys using sub-strings.
    pattern = f"<<{key}>>"
    # Replace occurrences of a particular sub-string with another.
    text = re.sub(pattern, value, text)
    return text


def slugify(value, allow_unicode=False):
    # Taken from
    # https://github.com/django/django/blob/master/django/utils/text.py
    """
    # Convert to ASCII if 'allow_unicode' is False. Convert spaces or repeated
    # dashes to single dashes. Remove characters that aren't alphanumerics,
    # underscores, or hyphens. Convert to lowercase. Also strip leading and
    # trailing whitespace, dashes, and underscores.
    """
    value = str(value)
    if allow_unicode:
        value = unicodedata.normalize('NFKC', value)
    else:
        value = (
            unicodedata.normalize('NFKD', value)
            .encode('ascii', 'ignore')
            .decode('ascii')
        )
    value = re.sub(r'[^\w\s-]', '', value.lower())
    return re.sub(r'[-\s]+', '-', value).strip('-_')


def recognize_file_name(key_value: str, file_name_key: str):
    # Search for the keys, then replace the pattern with the indicated key.
    final_name = file_name_key
    for key in key_value.keys():
        pattern = rf'{key}(?=[:\-_ .,\s]|$)'
        final_name = re.sub(pattern, key_value[key], final_name)
    return slugify(final_name)


def doc_generator(
        directory_template: str,
        key_value: dict,
        file_name_key: str,
        flag: int = 1):
    # Open the indicated directory.
    with open(directory_template, "r", encoding="utf-8") as input_file:
        text = input_file.read()
        input_file.close()
    filename = recognize_file_name(key_value, file_name_key)
    # Generates the file replacing data by data
    with open(f"./output/{filename}.md", "w+", encoding="utf-8") as input_file:
        for key in key_value.keys():
            text = loc_sub(text, key, key_value[key])
        input_file.write(text)
    if flag == '1':
        os.system(f"mdpdf -o ./output/{filename}.pdf ./output/{filename}.md")



from ez_docs.modules.doc_generation import loc_sub
from ez_docs.modules.doc_generation import slugify
from ez_docs.modules.doc_generation import recognize_file_name

key_value = {
    "NOME": "Wallace",
    "MATRICULA": "20990088"
}


def test_loc_sub():
    assert loc_sub("Ola <<Chave>>", "Chave", "Mundo") == "Ola Mundo"


def test_loc_sub_error():
    assert loc_sub("Ola <<Chave>>", "Chave", "Mundo") != "Ola Mudo"


def test_slugify():
    assert slugify("bruno#o:brabo. mago") == "brunoobrabo-mago"


def test_slugify_error():
    assert slugify("bruno#o:brabo. mago") != "bruno#o:brabo. mago"


def test_recognize_file_name():
    assert (
        recognize_file_name(key_value, "NOME_MATRICULA")
        ==
        "wallace_20990088"
    )


def test_recognize_file_name_error():
    assert (
        recognize_file_name(key_value, "NOME_MATRICULA")
        !=
        "Wallace_20990088"
    )



import argparse
import time
from ez_docs.modules.doc_generation import doc_generator
from ez_docs.modules.usable import verify_folder_output
from ez_docs.modules.usable import progress_bar
from ez_docs.modules.usable import clean_dir_md, zipfolder, CallCommand
from ez_docs.modules.data_cleaning import filter_data


def mk_docs(dict_args: dict):
    verify_folder_output()
    dataset = filter_data(dict_args['base_directory'], dict_args['constraint'])
    initial_time = time.time()
    for index in range(len(dataset)):
        doc_generator(
            dict_args['template_directory'],
            dataset[index], dict_args['file_name_pattern'],
            dict_args['flag']
        )
        progress_bar(index + 1, len(dataset), '#')

    print(f"Done.\nTotal generated objects: {len(dataset)}")
    print(f"Elapsed time: {time.time() - initial_time : 1.2f}s")

    if dict_args['flag'] == '1':
        clean_dir_md()

    if dict_args['zip'] == '1':
        print("Zipping files")
        zipfolder("output", "output")


def main():
    parser = argparse.ArgumentParser(
        description='Documents auto-generated.',
        add_help=False
    )
    parser.add_argument(
        '--about',
        nargs=0, help='About our project', action=CallCommand
    )
    parser.add_argument(
        '--architecture',
        nargs=0, help='Architecture stuff', action=CallCommand
    )
    parser.add_argument(
        '--help',
        nargs=0, help='How to use it', action=CallCommand
    )
    parser.add_argument(
        '--list',
        nargs=0, help='Command list', action=CallCommand
    )
    parser.add_argument(
        '--flag',
        help='Flag to output file extension. 0 for .md, 1 for .pdf',
        default='1'
    )
    parser.add_argument(
        '--zip',
        help='Define if the output folder is compacted. 1 for yes, 0 for no.',
        default='0'
    )
    parser.add_argument(
        '--constraint',
        help='Set your data set filtering constraints', nargs=1
    )
    parser.add_argument(
        'template_directory',
        help='Template dictionary.'
    )
    parser.add_argument(
        'base_directory',
        help='Database directory.'
    )
    parser.add_argument(
        'file_name_pattern',
        help='Output file pattern name.'
    )
    args = parser.parse_args()
    mk_docs(dict(args._get_kwargs()))


if __name__ == '__main__':
    main()



from ez_docs.modules.data_cleaning import filter_data, find_delimiter
from ez_docs.modules.data_cleaning import filter_format
import pytest
import pandas as pd
import tempfile

final_data = [
    {'nome': 'Bruno', 'idade': '18'},
    {'nome': 'Miguel', 'idade': '18'},
    {'nome': 'Gobbi', 'idade': '18'},
    {'nome': 'Igor', 'idade': '18'},
]

final_data2 = [
    {'nome': 'Bruno', 'idade': '18'},
    {'nome': 'Miguel', 'idade': '18'},
    {'nome': 'Gobbi', 'idade': '18'},
    {'nome': 'Igor', 'idade': '18'},
    {'nome': 'Igor', 'idade': '18'},
]


def test_find_delimiter():
    assert find_delimiter("test/teams1.csv") == ","
    assert find_delimiter("test/teams2.csv") == ";"
    assert find_delimiter("test/teams3.csv") == "\\"
    assert find_delimiter("test/teams4.csv") == "~"


def test_data_cleaning():
    assert filter_data("test/example.csv") == final_data


def test_data_cleaning_error():
    assert filter_data("test/example.csv") != final_data2


def test_filter_format():
    # Test with a CSV file
    with tempfile.NamedTemporaryFile(
        mode='w',
        delete=False,
        suffix='.csv'
    ) as temp:

        temp.write("col1,col2\n1,2\n3,4")
        temp.seek(0)
        result = filter_format(temp.name)
        assert isinstance(result, pd.DataFrame)

    # Test with a JSON file
    with tempfile.NamedTemporaryFile(
        mode='w',
        delete=False,
        suffix='.json'
    ) as temp:
        temp.write('{"col1": [1, 3], "col2": [2, 4]}')
        temp.seek(0)
        result = filter_format(temp.name)
        assert isinstance(result, pd.DataFrame)

    # Test with an invalid extension
    location = "path/to/invalid/file.invalid"
    with pytest.raises(Exception):
        filter_format(location)



import os
import glob
import argparse
from sys import argv, stdout
import importlib.resources
from ez_docs import cmd
import shutil


def verify_folder_output():
    # Check whether the specified path exists or not
    path = 'output/'
    exist = os.path.exists(path)
    if not exist:
        # Create a new directory because it does not exist
        os.makedirs(path)


def clean_dir_md():
    target_folder = 'output/'
    target_extension = 'md'
    # Find all the pathnames matching with the specified (target),
    # according to the rules used by the Unix.
    target_file_names = glob.glob(target_folder + '*.' + target_extension)
    # Delete the files.
    for file_name in target_file_names:
        os.remove(file_name)
    os.remove('mdpdf.log')


# idx - index
# amt - amount
def progress_bar(idx: int, amt: int, char: str, final_length: int = 25):
    stdout.write("\033[F\033[K\033[F\033[K\033[F\033[K")
    out = f"\033[93m({idx}/{amt})\033[0m {idx * 100/amt: 1.2f}% [\033[96m"
    print(
        out, end=''
    )
    charline = "".join([char for _ in range(round(final_length*idx/amt))])
    print(charline + "\033[0m]")


def zipfolder(foldername, target_dir):
    folder_to_zip = foldername
    output_zip = target_dir

    shutil.make_archive(output_zip, "zip", folder_to_zip)
    shutil.rmtree("output")


class CallCommand(argparse.Action):
    def __call__(self, parser, namespace, values, option_string):
        # Catch the last argument given in the sys.arg
        command = argv[-1].lstrip('-')

        # Open the txt response
        path = importlib.resources.files(cmd)

        with open(file=f'{path}/{command}.txt', encoding='utf-8') as f:
            data = f.read()
            print(data)
        # Close the parser
        parser.exit()



from ez_docs.main import mk_docs, main
import shutil
import os
import argparse

# "template.md example.csv name_idade"

example = {
    "template_directory": "test/template.md",
    "base_directory": "test/example.csv",
    "file_name_pattern": "nome_idade",
    "flag": 1,
    'zip': 0,
    "constraint": ""
}


def test_mk_docs():
    mk_docs(example)
    x = len(os.listdir('output'))
    shutil.rmtree('output')
    assert x == 4


def test_mk_docs_error():
    mk_docs(example)
    x = len(os.listdir('output'))
    shutil.rmtree('output')
    assert x != 3


def test_main(monkeypatch):
    def mock_parse_args(self):
        return argparse.Namespace(
            zip=0,
            flag=1,
            template_directory='template.md',
            base_directory='dados.csv', file_name_pattern='NOME_MATRICULA',
            _get_kwargs=lambda: {
                'flag': 1,
                'template_directory': 'test/template.md',
                'base_directory': 'test/example.csv',
                'file_name_pattern': 'nome_idade',
                'zip': 0,
                "constraint": ""
            }
        )
    monkeypatch.setattr(argparse.ArgumentParser, 'parse_args', mock_parse_args)
    main()
    x = len(os.listdir('output'))
    shutil.rmtree('output')
    assert x == 4


def test_main_error(monkeypatch):
    def mock_parse_args(self):
        return argparse.Namespace(
            flag=1,
            zip=0,
            template_directory='template.md',
            base_directory='dados.csv', file_name_pattern='NOME_MATRICULA',

            _get_kwargs=lambda: {
                'flag': 1,
                'template_directory': 'test/template.md',
                'base_directory': 'test/example.csv',
                'file_name_pattern': 'nome_idade',
                'zip': '0',
                "constraint": ""
            }
        )
    monkeypatch.setattr(argparse.ArgumentParser, 'parse_args', mock_parse_args)
    main()
    x = len(os.listdir('output'))
    shutil.rmtree('output')
    assert x != 3



from ez_docs.modules.usable import progress_bar, zipfolder, clean_dir_md
import shutil
import glob
import os


def test_progress_bar(capsys):
    progress_bar(4, 10, "#", 10)
    out, err = capsys.readouterr()
    out = out[
        len("\x1b[93m(1/10)\x1b[0m  10.00% [\x1b[96m"): -len("\x1b[0m]\n")
    ]
    assert out == "####"


def test_progress_bar_error(capsys):
    progress_bar(8, 10, "#", 10)
    out, err = capsys.readouterr()
    out = out[
        len("\x1b[93m(1/10)\x1b[0m  10.00% [\x1b[96m"): -len("\x1b[0m]\n")
    ]
    assert out != "#####"


def test_zipfolder():
    foldername = "testing_folder"
    target_dir = "test_zip"

    # Create a test folder
    os.makedirs(foldername)
    os.makedirs('output')

    # Call the zipfolder function
    zipfolder(foldername, target_dir)

    # Check if the zip file was created
    assert os.path.exists(target_dir + ".zip")

    # Clean up
    os.remove(target_dir + ".zip")
    shutil.rmtree(foldername)


def test_clean_dir_md():

    os.makedirs('output')

    # Create some test .md files
    with open("output/test1.md", "w") as f:
        f.write("Test content")
    with open("output/test2.md", "w") as f:
        f.write("Test content")
    with open("mdpdf.log", "w") as f:
        f.write("Test log content")

    # Call the clean_dir_md function
    clean_dir_md()

    # Check if the files and the log were deleted
    assert not glob.glob("output/*.md")
    assert not os.path.exists("mdpdf.log")



import pandas as pd


# Finds out which non-alphanumeric character separates a given csv file
def find_delimiter(location: str) -> str:
    delimiters_dict = {}
    file = open(location).read()
    temp_list = [
        char for char in file if not (
            char.isalpha() or char.isspace() or char.isdigit()
        )
    ]
    delimiters_list = list(set(temp_list))
    for line in file.split("\n"):
        for delimiter in delimiters_list:
            if delimiter not in delimiters_dict.keys():
                delimiters_dict[delimiter] = []
            delimiters_dict[delimiter].append(len(line.split(delimiter)))
    for delimiter in delimiters_dict.keys():
        delimiters_dict[delimiter] = len(set(delimiters_dict[delimiter]))
    delimiters_dict = sorted(delimiters_dict.items(), key=lambda item: item[1])
    return delimiters_dict[0][0]


# Returns a pandas DataFrame according to an extension
def filter_format(location: str) -> pd.DataFrame:
    extension = location.split(".")[-1]
    if extension == "csv":
        return pd.read_csv(location, delimiter=find_delimiter(location))
    elif extension == "html":
        return pd.read_html(location)
    elif extension == "json":
        return pd.read_json(location)
    elif extension == "xlsx":
        return pd.read_excel(location)
    elif extension == "xml":
        return pd.read_xml(location)
    else:
        raise Exception(
                f"""
                \033[0;31mThe extension '{extension}' is not accepted.\n
                Valid: csv, html, json, xlsx, xml.\033[0m
                """
            )


# Converts the data_set to the format specified in the architecture
def filter_data(location: str, constraint: str = "") -> list:
    try:
        start_data = filter_format(location)
        if True in start_data.isna().values:
            print(
                    """
                    \033[0;33mWarning: Your dataset file has incomplete lines.
                    These ones will be ignored.\033[0m
                    """
                )
        start_data.dropna(inplace=True)
        start_data.drop_duplicates(inplace=True)
        if constraint:
            start_data.query(constraint[0], inplace=True)
        return [
            {
                col: str(start_data.iloc[line][col])
                for col in start_data.columns
            } for line in range(len(start_data))
        ]
    except FileNotFoundError:
        raise Exception(
            """
                \033[0;31mThe dataset file was not found.\n
                Please, make sure you have typed a correct file name.\033[0m
            """
        )



from setuptools import setup, find_packages

with open('requirements.txt') as f:
    requirements = f.readlines()

with open("README.md", "r") as fh:
    readme = fh.read()

long_description = readme

cmds = ["cmd/*.txt"]

setup(
    name='ez_docs',
    version='1.1.0',
    author="""
        Bruno Ribeiro,
        Bruno Martins,
        Diógenes Dantas,
        Igor Penha,
        Lucas Gobbi e Rafael Nobre
    """,
    author_email='ezdocsteam@gmail.com',
    url='https://github.com/fga-eps-mds/2022-2-ez-docs',
    description='A python package to quick doc generation',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='MIT',
    keywords=['ez_docs', 'ez-docs'],
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    package_data={'ez_docs': cmds},
    entry_points={
            'console_scripts': [
                'ez-docs=ez_docs.main:main'
            ]
    },
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),

    install_requires=requirements
)
